#!/usr/bin/env python
import Numeric, arrayfrombuffer, mmap, os, stat, unittest

def maparray(file, typecode='l', shape=None):
    """mmap a file as an array, either given a file object or a filename.
    
    Returns the array, the mmap object (so you can call flush
    explicitly to ensure your changes have gotten copied out to disk),
    and the open file.

    """
    
    if not hasattr(file, 'fileno'): file = open(file, 'rb+')
    fn = file.fileno()
    m = mmap.mmap(fn, os.fstat(fn)[stat.ST_SIZE])
    a = arrayfrombuffer.arrayfrombuffer(m, typecode)
    if shape is not None: a.shape = shape
    return a, m, file

class maparraytest(unittest.TestCase):
    def setUp(self):
        self.x = Numeric.arange(10) / 2.0
        open('tmp.foo', 'wb').write(self.x.tostring())
    def tearDown(self):
        os.unlink('tmp.foo')
    def testx(self):
        self.failUnless(Numeric.alltrue(self.x ==
                                        Numeric.array([0, 0.5, 1, 1.5, 2,
                                                       2.5, 3, 3.5, 4, 4.5])))
    def testread(self):
        y, _, _ = maparray('tmp.foo', 'd')
        self.failUnless(Numeric.alltrue(self.x == y))
    def testwrite(self):
        y, mapobj, _ = maparray('tmp.foo', 'd')
        y[1] = 37
        mapobj.flush()
        z, _, _ = maparray('tmp.foo', 'd')
        self.failUnless(Numeric.alltrue(z ==
                                        Numeric.array([0, 37, 1, 1.5, 2,
                                                       2.5, 3, 3.5, 4, 4.5])))
        self.failUnless(Numeric.alltrue(z == y))
    def testimplicitflush(self):
        y, _, _ = maparray('tmp.foo', 'd')
        y[1] = 37
        del y, _  # trigger implicit flush as refcounts go to 0
        z, _, _ = maparray('tmp.foo', 'd')
        self.failUnless(Numeric.alltrue(z ==
                                        Numeric.array([0, 37, 1, 1.5, 2,
                                                       2.5, 3, 3.5, 4, 4.5])))
    def testnewlines(self):
        # If we forgot to open the file in binary mode, some bytes would
        # get mangled on some systems.
        x = Numeric.arange(258).astype('b')  # bytes
        x[-2:] = (13, 10)  # CRLF
        open('tmp.foo', 'wb').write(x.tostring())
        y, _, _ = maparray('tmp.foo', 'b')
        self.failUnless(Numeric.alltrue(x == y))
    def testshape(self):
        xx = [[1, 3], [5, 7]]
        open('tmp.foo', 'wb').write(Numeric.array(xx).tostring())
        self.assertEqual(maparray('tmp.foo', shape=(2, 2))[0].tolist(), xx)
    # There are a bunch of features that are sort of hard to test, because
    # they're performance-oriented and require big data files or because
    # they involve transcending system limitations that may not be present.
    # For example:
    # - mapping an array should return instantly, even for large files
    # - reading the contents of a mapped array shouldn't take much more time
    #   than reading in the corresponding file with ordinary read() calls
    # - it should be possible to have arrays much larger than RAM
    #   (this is obviously hard to test if you have more than a moby of RAM)
    #   and operate on them reasonably efficiently.
    # - reading or writing a small part of the array should be fast

if __name__ == "__main__": unittest.main()
